// import echarts from 'echarts'
import chartUtils from './chartUtils'
// import './echarts-tooltip-carousel'
/* 是否开启显示 */
const selectedShow = function (bool1, bool2) {
    let reBool = null;
    if (typeof (bool1) != 'boolean') {
        bool1 = bool1 == '是' ? true : false;
    }
    if (typeof (bool2) != 'boolean') {
        bool2 = bool2 == '是' ? true : false;
    }

    if (bool1 && bool2) {
        reBool = true;
    } else {
        reBool = false;
    }
    return reBool;
};
/* 创建纬度是否显示 */
const switchShow = function (styles, name, bool2) {
    let bool1 = chartUtils.getStyleValue(styles, name);

    let reBool = null;
    if (typeof (bool1) != 'boolean') {
        bool1 = bool1 == '是' ? true : false;
    }
    if (typeof (bool2) != 'boolean') {
        bool2 = bool2 == '是' ? true : false;
    }

    if (bool1 && bool2) {
        reBool = true;
    } else {
        reBool = false;
    }
    return reBool;
};

/* 设置图表阴影 */
const setShadow = function (globalData, styles) {
    // 是否开启图表阴影
    let showShader = chartUtils.getStyleValue(styles, "showShader");
    let shadowSetting = chartUtils.getShadowSetting();
    // 如果不开启图表阴影，则将阴影颜色改为全透明
    if (showShader !== true) {
        shadowSetting.shadowColor = "rgba(0,0,0,0)";
    }

    // 标题文字阴影
    globalData.title.textStyle.textShadowColor = shadowSetting.shadowColor;
    globalData.title.textStyle.textShadowOffsetX =
        shadowSetting.shadowOffsetX;
    globalData.title.textStyle.textShadowOffsetY =
        shadowSetting.shadowOffsetY;
    globalData.title.textStyle.textShadowBlur = shadowSetting.shadowBlur;

    // 横轴阴影（轴线阴影和文字阴影）
    // 轴线阴影
    globalData.xAxis.axisLine.lineStyle.shadowColor =
        shadowSetting.shadowColor;
    globalData.xAxis.axisLine.lineStyle.shadowOffsetX =
        shadowSetting.shadowOffsetX;
    globalData.xAxis.axisLine.lineStyle.shadowOffsetY =
        shadowSetting.shadowOffsetY;
    globalData.xAxis.axisLine.lineStyle.shadowBlur = shadowSetting.shadowBlur;
    // 轴文字阴影
    globalData.xAxis.axisLabel.textShadowColor = shadowSetting.shadowColor;
    globalData.xAxis.axisLabel.textShadowOffsetX =
        shadowSetting.shadowOffsetX;
    globalData.xAxis.axisLabel.textShadowOffsetY =
        shadowSetting.shadowOffsetY;
    globalData.xAxis.axisLabel.textShadowBlur = shadowSetting.shadowBlur;

    // 纵轴阴影（轴线阴影和文字阴影）
    // 轴线阴影
    globalData.yAxis.axisLine.lineStyle.shadowColor =
        shadowSetting.shadowColor;
    globalData.yAxis.axisLine.lineStyle.shadowOffsetX =
        shadowSetting.shadowOffsetX;
    globalData.yAxis.axisLine.lineStyle.shadowOffsetY =
        shadowSetting.shadowOffsetY;
    globalData.yAxis.axisLine.lineStyle.shadowBlur = shadowSetting.shadowBlur;
    // 轴文字阴影
    globalData.yAxis.axisLabel.textShadowColor = shadowSetting.shadowColor;
    globalData.yAxis.axisLabel.textShadowOffsetX =
        shadowSetting.shadowOffsetX;
    globalData.yAxis.axisLabel.textShadowOffsetY =
        shadowSetting.shadowOffsetY;
    globalData.yAxis.axisLabel.textShadowBlur = shadowSetting.shadowBlur;
};

/* 设置初始化动画 */
const setInitAnimation = function (obj, styles) {
    obj.animation = chartUtils.getStyleValue(styles, "animation");
    obj.animationEasing = chartUtils.getStyleValue(
        styles,
        "animationEffect"
    );
    obj.animationDuration = chartUtils.getStyleValue(
        styles,
        "animationDuration"
    );
};

/* 创建背景 */
const createBackgroundGrid = function (styles) {
    let grid = {
        left: chartUtils.getStyleValue(styles, "gridLeft"),
        top: chartUtils.getStyleValue(styles, "gridTop"),
        right: chartUtils.getStyleValue(styles, "gridRight"),
        bottom: chartUtils.getStyleValue(styles, "gridBottom"),
        containLabel: true,
        borderWidth: 0
    };
    return grid;
};
/* 创建维度轴（X轴）背景网格面 */
const createSplitArea = function (styles) {
    let showGrid = chartUtils.getStyleValue(styles, "showGrid");
    var splitArea = {
        show: this.switchShow(styles, 'measureAxis', showGrid),
        areaStyle: {
            color: [
                chartUtils.getStyleValue(styles, "gridColor1", "color"),
                chartUtils.getStyleValue(styles, "gridColor2", "color")
            ]
        }
    };
    return splitArea;
};

/* 创建参考线 */
const createMarkLine = function (styles, isAxisX) {
    let maxReferenceLineType = chartUtils.getStyleValue(
        styles,
        "maxReferenceLineType"
    );
    let minReferenceLineType = chartUtils.getStyleValue(
        styles,
        "minReferenceLineType"
    );
    var markLine = {
        silent: true,
        symbol: "none",
        label: {
            normal: {
                show: false
            }
        },
        data: [{
                lineStyle: {
                    normal: {
                        type: chartUtils.convertLineTypeToString(maxReferenceLineType),
                        color: chartUtils.getStyleValue(
                            styles,
                            "maxReferenceLineColor",
                            "color"
                        ),
                        width: chartUtils.getStyleValue(
                            styles,
                            "maxReferenceLineWidth"
                        )
                    }
                }
            },
            {
                lineStyle: {
                    normal: {
                        type: chartUtils.convertLineTypeToString(minReferenceLineType),
                        color: chartUtils.getStyleValue(
                            styles,
                            "minReferenceLineColor",
                            "color"
                        ),
                        width: chartUtils.getStyleValue(
                            styles,
                            "minReferenceLineWidth"
                        )
                    }
                }
            }
        ]
    };
    let maxValue = chartUtils.getStyleValue(
        styles,
        "maxReferenceLineValue"
    );
    let minValue = chartUtils.getStyleValue(
        styles,
        "minReferenceLineValue"
    );
    if (isAxisX) {
        markLine.data[0].xAxis = maxValue;
        markLine.data[1].xAxis = minValue;
    } else {
        markLine.data[0].yAxis = maxValue;
        markLine.data[1].yAxis = minValue;
    }
    return markLine;
};

/* 创建维度轴（X轴）刻度样式 */
const createDimensionAxisTick = function (styles) {
    let showDimensionAxisTickLine = chartUtils.getStyleValue(
        styles,
        "showDimensionAxisTickLine"
    );
    var axisTick = {
        show: this.switchShow(styles, 'dimensionAxis', showDimensionAxisTickLine),
        length: chartUtils.getStyleValue(
            styles,
            "dimensionAxisTickLineLength"
        ),
        lineStyle: {
            color: chartUtils.getStyleValue(
                styles,
                "dimensionAxisTickLineColor",
                "color"
            ),
            width: chartUtils.getStyleValue(
                styles,
                "dimensionAxisTickLineWidth"
            )
        }
    };
    return axisTick;
};

/* 创建维度轴（X轴）背景网格线 */
const createDimensionAxisSplitLine = function (styles) {
    let showDimensionAxisSplitLine = chartUtils.getStyleValue(
        styles,
        "showDimensionAxisSplitLine"
    );
    let dimensionAxisSplitLineType = chartUtils.getStyleValue(
        styles,
        "dimensionAxisSplitLineType"
    );
    var splitLine = {
        show: this.switchShow(styles, 'dimensionAxis', showDimensionAxisSplitLine),
        lineStyle: {
            color: chartUtils.getStyleValue(
                styles,
                "dimensionAxisSplitLineColor",
                "color"
            ),
            width: chartUtils.getStyleValue(
                styles,
                "dimensionAxisSplitLineWidth"
            ),
            type: chartUtils.convertLineTypeToString(dimensionAxisSplitLineType)
        }
    };
    return splitLine;
};
/* 创建维度轴（X轴）刻度线样式 */
const createDimensionAxisLine = function (styles) {
    let showDimensionAxisLine = chartUtils.getStyleValue(
        styles,
        "showDimensionAxisLine"
    );
    var axisLine = {
        show: this.switchShow(styles, 'dimensionAxis', showDimensionAxisLine),
        lineStyle: {
            color: chartUtils.getStyleValue(
                styles,
                "dimensionAxisLineColor",
                "color"
            ),
            width: chartUtils.getStyleValue(styles, "dimensionAxisLineWidth")
        }
    };
    return axisLine;
};
/* 创建指标轴（Y轴）分割线 背景网格线 */
const createMeasureAxisSplitLine = function (styles) {
    let showMeasureAxisSplitLine = chartUtils.getStyleValue(
        styles,
        "showMeasureAxisSplitLine"
    );
    let measureAxisSplitLineType = chartUtils.getStyleValue(
        styles,
        "measureAxisSplitLineType"
    );
    var splitLine = {
        show: this.switchShow(styles, 'measureAxis', showMeasureAxisSplitLine),
        lineStyle: {
            color: chartUtils.getStyleValue(
                styles,
                "measureAxisSplitLineColor",
                "color"
            ),
            width: chartUtils.getStyleValue(
                styles,
                "measureAxisSplitLineWidth"
            ),
            type: chartUtils.convertLineTypeToString(measureAxisSplitLineType)
        }
    };
    return splitLine;
};
/* 创建指标轴（Y轴）刻度样式 */
const createMeasureAxisTick = function (styles) {
    let showMeasureAxisTickLine = chartUtils.getStyleValue(
        styles,
        "showMeasureAxisTickLine"
    );
    var axisTick = {
        show: this.switchShow(styles, 'measureAxis', showMeasureAxisTickLine),
        length: chartUtils.getStyleValue(
            styles,
            "measureAxisTickLineLength"
        ),
        lineStyle: {
            color: chartUtils.getStyleValue(
                styles,
                "measureAxisTickLineColor",
                "color"
            ),
            width: chartUtils.getStyleValue(
                styles,
                "measureAxisTickLineWidth"
            )
        }
    };
    return axisTick;
};
/* 创建指标轴（Y轴）刻度线样式 */
const createMeasureAxisLine = function (styles) {
    let showMeasureAxisLine = chartUtils.getStyleValue(
        styles,
        "showMeasureAxisLine"
    );
    var axisLine = {
        show: this.switchShow(styles, 'measureAxis', showMeasureAxisLine),
        lineStyle: {
            color: chartUtils.getStyleValue(
                styles,
                "measureAxisLineColor",
                "color"
            ),
            width: chartUtils.getStyleValue(styles, "measureAxisLineWidth")
        }
    };
    return axisLine;
};
/* 创建 纬度轴（X轴）轴标签 标签文字样式 */
const createDimensionAxisLabel = function (styles) {
    let showDimensionAxisLabel = chartUtils.getStyleValue(
        styles,
        "showDimensionAxisLabel"
    );
    let fontRotate = chartUtils.getStyleValue(
        styles,
        "dimensionAxisLabelFontRotate"
    );

    var axisLabel = {
        show: this.switchShow(styles, 'dimensionAxis', showDimensionAxisLabel),
        textStyle: {
            color: chartUtils.getStyleValue(
                styles,
                "dimensionAxisLabelForeground",
                "color"
            ),
            fontSize: chartUtils.getStyleValue(
                styles,
                "dimensionAxisLabelFontSize"
            ),
            fontFamily: chartUtils.getStyleValue(styles, "fontfamily"),
            fontWeight: chartUtils.getStyleValue(
                styles,
                "dimensionAxisLabelFontWeight"
            ),
            fontStyle: chartUtils.convertRotateToFontStyle(fontRotate)
        }
        //rotate: 20
    };
    return axisLabel;

}
/* 创建 纬度轴（Y轴）轴标签 标签文字样式 */
const createMeasureAxisLabel = function (styles) {
    let showMeasureAxisLabel = chartUtils.getStyleValue(
        styles,
        "showMeasureAxisLabel"
    );
    let fontRotate = chartUtils.getStyleValue(
        styles,
        "measureAxisLabelFontRotate"
    );

    var axisLabel = {
        show: this.switchShow(styles, 'measureAxis', showMeasureAxisLabel),
        textStyle: {
            color: chartUtils.getStyleValue(
                styles,
                "measureAxisLabelForeground",
                "color"
            ),
            fontSize: chartUtils.getStyleValue(
                styles,
                "measureAxisLabelFontSize"
            ),
            fontFamily: chartUtils.getStyleValue(styles, "fontfamily"),
            fontWeight: chartUtils.getStyleValue(
                styles,
                "measureAxisLabelFontWeight"
            ),
            fontStyle: chartUtils.convertRotateToFontStyle(fontRotate)
        }
        //rotate: 20
    };
    return axisLabel;
};
/* 创建标题 */
const createTitle = function (styles) {
    let titleAlignment = chartUtils.getStyleValue(
        styles,
        "titleAlignment"
    );
    let fontRotate = chartUtils.getStyleValue(styles, "titleRotate");

    // 根据标题对齐方式，计算 ecahrts title 的 left、right、textAlign
    let eLeft = null;
    let eRight = null;
    let eTextAlign = null;
    if (titleAlignment == "居左") {
        eLeft = "0%";
    } else if (titleAlignment == "居中") {
        eLeft = "50%";
        eTextAlign = "center";
    } else {
        eRight = "0%";
    }

    // 根据标题的水平和竖直偏移，计算 echarts title 的 padding
    let ePadding = [0, 0, 0, 0];
    let titleHorOffset = chartUtils.getStyleValue(
        styles,
        "titleHorOffset"
    );
    let titleVerOffset = chartUtils.getStyleValue(
        styles,
        "titleVerOffset"
    );
    ePadding[0] = titleVerOffset;
    ePadding[3] = titleHorOffset;

    var title = {
        show: chartUtils.getStyleValue(styles, "showTitle"),
        text: chartUtils.getStyleValue(styles, "titleContent"),
        left: eLeft,
        right: eRight,
        textAlign: eTextAlign,
        padding: ePadding,
        textStyle: {
            color: chartUtils.getStyleValue(
                styles,
                "titleForeground",
                "color"
            ),
            fontSize: chartUtils.getStyleValue(styles, "titleFontSize"),
            fontFamily: chartUtils.getStyleValue(styles, "fontfamily"),
            fontWeight: chartUtils.getStyleValue(styles, "titleFontWeight"),
            fontStyle: chartUtils.convertRotateToFontStyle(fontRotate)
        }
    };
    return title;
};
/* 创建tooltip */
const createTooltip = function (styles) {
    let showTooltip = chartUtils.getStyleValue(styles, "showTooltip");
    let fontRotate = chartUtils.getStyleValue(styles, "tooltipFontRotate");
    let tooltipGridIndicatorStyle = chartUtils.getStyleValue(styles, "tooltipGridIndicatorStyle");
    var tooltip = {
        show: this.switchShow(styles, 'tooltip', showTooltip),
        showContent: showTooltip,
        trigger: "axis",
        axisPointer: {
            type: "line",
            z: -10000,
            lineStyle: {
                color: chartUtils.getStyleValue(
                    styles,
                    "tooltipGridIndicatorColor",
                    "color"
                ),
                width: chartUtils.getStyleValue(
                    styles,
                    "tooltipGridIndicatorWidth"
                ),
                type: chartUtils.convertLineTypeToString(tooltipGridIndicatorStyle)
            }
        },
        //设置toolTip的边框，颜色和字体颜色
        backgroundColor: chartUtils.getStyleValue(
            styles,
            "tooltipBackground",
            "color"
        ),
        borderColor: chartUtils.getStyleValue(
            styles,
            "tooltipBackgroundBorderColor",
            "color"
        ),
        borderWidth: chartUtils.getStyleValue(
            styles,
            "tooltipBackgroundBorderThickness"
        ),
        textStyle: {
            color: chartUtils.getStyleValue(
                styles,
                "tooltipForeground",
                "color"
            ),
            fontSize: chartUtils.getStyleValue(styles, "tooltipFontSize"),
            fontFamily: chartUtils.getStyleValue(styles, "fontfamily"),
            fontWeight: chartUtils.getStyleValue(
                styles,
                "tooltipFontWeight"
            ),
            fontStyle: chartUtils.convertRotateToFontStyle(fontRotate)
        }
    };
    return tooltip;
};
/* 创建数量轴的 分割段数*/
const createSplitNum = function (minV, maxV, splitNumberV) {
    let min = null;
    let max = null;
    let splitNumber = 5;
    if (splitNumberV) {
        splitNumber = splitNumberV;
    }
    //当最大值有值的时，判断处理
    if (maxV) {
        if (min >= maxV) {
            min = null;
        } else {
            min = minV
            max = maxV
        }
    } else {
        //当最大值无值的时，判断处理
        if (min >= maxV) {
            min = null;
            max = null;
        } else {
            min = minV
            max = maxV
        }
    }
    return {
        min,
        max,
        splitNumber
    }
};
/* 图例显示位置 */
const legendPosition = function (pos, offsetX, offsetY) {
    let position = {};
    // vertical 垂直，水平 horizontal
    if (pos == '顶部') {
        position = {
            left: offsetX,
            top: offsetY,
            orient: 'horizontal'
        }
    } else if (pos == '底部') {
        position = {
            bottom: offsetY,
            left: offsetX,
            orient: 'horizontal'
        }
    } else if (pos == '左部') {
        position = {
            left: offsetX,
            top: offsetY,
            orient: 'vertical'
        }
    } else if (pos == '右部') {
        position = {
            right: offsetX,
            top: offsetY,
            orient: 'vertical'
        }
    }
    return position;
};
//坐标指示器


export default {
    selectedShow,
    switchShow,
    setShadow,
    setInitAnimation,
    createBackgroundGrid,
    createSplitArea,
    createMarkLine,
    createDimensionAxisTick,
    createDimensionAxisSplitLine,
    createDimensionAxisLine,
    createMeasureAxisSplitLine,
    createMeasureAxisTick,
    createMeasureAxisLine,
    createTitle,
    createTooltip,
    createDimensionAxisLabel,
    createMeasureAxisLabel,
    createSplitNum,
    legendPosition
}