import echarts from 'echarts'



/* 通用方法 数组深拷贝 */
const deepClone = function (obj) {
    var objClone = Array.isArray(obj) ? [] : {};
    if (obj && typeof obj === "object") {
        for (var key in obj) {
            if (obj.hasOwnProperty(key)) {
                if (obj[key] && typeof obj[key] === "object") {
                    objClone[key] = this.deepClone(obj[key]);
                } else {
                    objClone[key] = obj[key];
                }
            }
        }
    }
    return objClone;
}
/* 获取对象中最大值、最小值 */
/* 
 * array 数据
 * property 对比的属性名,
 * type min 最小值，max 最大值
 */
const getMaxOrMin = function (array, property, type) {
    let val = null;
    if (type == "max") {
        val = Math.max.apply(Math, array.map((o) => {
            return o[property]
        }))
    } else if (type == "min") {
        val = Math.min.apply(Math, array.map((o) => {
            return o[property]
        }))
    }
    return val;
}

/* 通用转换 获取颜色结束点坐标 */
const getDataValue = function (dataMax, dataMin, sampleMax, sampleMin, dataCurrent) {
    if (dataMax == dataMin) {
        return sampleMax;
    }
    if (dataCurrent <= dataMin) {
        return sampleMin;
    }
    if (dataCurrent >= dataMax) {
        return sampleMax;
    }
    return (sampleMax - sampleMin) / (dataMax - dataMin) * (dataCurrent - dataMin) + sampleMin;
}

/* 通用转换 获取颜色结束点坐标 */
const getColorEndPoint = function (startX, startY, origanX, origanY, angle) {
    let endX = (startX + (origanX - startX) * Math.cos(angle * Math.PI / 180) - (origanY - startY) * Math.sin(angle * Math.PI / 180));
    let endY = (startY + (origanX - startX) * Math.sin(angle * Math.PI / 180) + (origanY - startY) * Math.cos(angle * Math.PI / 180));
    endX = +(endX.toFixed(6));
    endY = +(endY.toFixed(6));
    if (endX < 0 || endY < 0) {
        startX += 1;
        startY += 1;
        endX += 1;
        endY += 1;
    }
    let point = {
        x1: startX,
        y1: startY,
        x2: endX,
        y2: endY
    }
    return point;
}

//文本属性颜色
const getTextColor = function (propertyValue) {
    let result = "red";
    if (propertyValue == null) {
        result = "red";
    }
    if (propertyValue.type == "single") {

        result = propertyValue.color;
    } else if (propertyValue.type == "gradient") {

        result = "background-image: -webkit-gradient(linear, 0 13, 0 top, from(" + colorRgba(propertyValue.colors[0].color) + "), to(" + colorRgba(propertyValue.colors[1].color) + "));-webkit-background-clip: text;-webkit-text-fill-color: transparent;"
    }
    return result;
}

//图片拉伸
const imageStretch = function (propertyValue) {
    let result = "";
    if (propertyValue == "Fill") {
        result = "width:100%;height:100%;";
    }
    if (propertyValue == "None") {
        result = "overflow: hidden;"
    }
    if (propertyValue == "Uniform") {
        result = "width: auto;height: auto;max-width: 100%;max-height: 100%;"
    }
    return result;
}

/* 获取api字段名称 */
const getApiFieldNameFromMapping = function (dataType, api, chartFieldName) {
    if (dataType.toLowerCase() == "api接口") {
        let apiField = api.resultChartMapping.find(
            s => s.chart.toLowerCase() == chartFieldName.toLowerCase()
        );
        return apiField.api;
    } else {
        return "";
    }
}

/* 通用转换 将属性值改为颜色 */
const getColor = function (propertyValue, type) {
    let result = "red";
    if (propertyValue == null) {
        result = "red";
    }
    if (propertyValue.type == "single") {
        result = colorRgba(propertyValue.color);
    } else if (propertyValue.type == "gradient") {
        if (type) {
            if (type == "background") {
                result = `linear-gradient(${
                    propertyValue.direction
                    }deg,${propertyValue.colors[0].color} ${propertyValue
                        .colors[0].offset * 100}%,${
                    propertyValue.colors[1].color
                    } ${propertyValue.colors[1].offset * 100}%)`;
            } else {
                let newDirection = Math.floor(propertyValue.direction / 360 + 0.5);
                propertyValue.direction -= newDirection * 360;
                let point = getColorEndPoint(0, 0, 0, 1, propertyValue.direction);
                result = new echarts.graphic.LinearGradient(point.x1, point.y1, point.x2, point.y2, [{
                        offset: propertyValue.colors[0].offset,
                        color: propertyValue.colors[0].color
                    },
                    {
                        offset: propertyValue.colors[1].offset,
                        color: propertyValue.colors[1].color
                    }
                ]);
            }
        } else {
            result = `linear-gradient(${
                propertyValue.direction
                }deg,${propertyValue.colors[0].color} ${propertyValue
                    .colors[0].offset * 100}%,${
                propertyValue.colors[1].color
                } ${propertyValue.colors[1].offset * 100}%)`;
        }
    } else {
        result = "red";
    }
    return result;
}

/* 数值保留小数位，返回值的类型是字符串 */
const getNumberString = function (number, decimal) {
    if (decimal == -1) {
        return number;
    }
    let result = number.toFixed(decimal);
    return result;
};

/* 获取数值文字的整数部分 */
const getIntegerPart = function (numberString) {
    let result = "0";
    let index = numberString.indexOf(".");
    if (index != -1) {
        result = numberString.slice(0, index);
    }
    return result;
};

/* 获取数值文字的小数部分 */
const getDecimalPart = function (numberString) {
    let result = "";
    let index = numberString.indexOf(".");
    if (index != -1) {
        result = numberString.slice(index + 1);
    }
    return result;
};

/* 获取千位符格式（整数字符串） */
const toThousandsFromIntegerString = function (integerString) {
    let num = integerString;
    let result = '';
    while (num.length > 3) {
        result = ',' + num.slice(-3) + result;
        num = num.slice(0, num.length - 3);
    }
    if (num) {
        result = num + result;
    }
    return result;
};

/* 获取千位符格式（数值字符串） */
const toThousandsFromNumberString = function (numberString) {
    let integerPart = getIntegerPart(numberString);
    let decimalPart = getDecimalPart(numberString);
    let thousandsPart = toThousandsFromIntegerString(integerPart);
    let result = thousandsPart + '.' + decimalPart;
    return result;
};

/* 获取阴影设置参数 */
const getShadowSetting = function () {
    return {
        shadowColor: 'rgba(0,0,0,0,5)',
        shadowOffsetX: 6,
        shadowOffsetY: 3,
        shadowBlur: 1
    };
};

/* 通用转换 位置转为属性字符串 */
const convertPositionToString = function (position) {
    if (position == "居中") {
        return "middle";
    } else if (position == "居左") {
        return "left";
    } else if (position == "居右") {
        return "right";
    } else if (position == "居上") {
        return "top";
    } else if (position == "居下") {
        return "bottom";
    }
}

/* 通用转换 是否旋转转为字体样式 */
const convertRotateToFontStyle = function (fontRotate) {
    if (fontRotate == "是") {
        return "italic";
    } else {
        return "normal";
    }
}

/* 通用转换 是否转换为true/false */
const convertYesNoToBoolean = function (yesno) {
    if (yesno == "是") {
        return true;
    } else return false;
}

/* 通用转换 线型转换为字符串 */
const convertLineTypeToString = function (lineType) {
    if (lineType == "实线") {
        return "solid";
    } else if (lineType == "虚线") {
        return "dashed";
    } else if (lineType == "点线") {
        return "dotted";
    }
}

/* 通用转换 浮点型转换为百分数 */
const convertFloatValueToPercentValue = function (floatValue) {
    return floatValue * 100 + "%";
}

/* 通用转换 值标签样式转换为是否可见 */
const convertValueStyleToVisibility = function (valueStyle) {
    if (valueStyle == "无") {
        return false;
    }
    return true;
}

/* 获取属性值 */
const getStyleValue = function (styles, name, type1, type2) {
    var result = styles.find(s => {
        return s.name.toLowerCase() == name.toLowerCase();
    });
    if (type1) {
        if (type1.toLowerCase() == "color") {
            return getColor(result.value, type2);
        }
    }
    return result.value;
}
/* 获取一条属性 */
const getStyleObj = function (styles, name) {
    var result = styles.find(s => {
        return s.name.toLowerCase() == name.toLowerCase();
    });

    return result;
}

/* 根据category1名称，获取该类别下的样式集合 */
const getStylesFromCategory = function (styles, category1) {
    var result = [];
    for (var i = 0; i < styles.length; i++) {
        if (styles[i].category1.toLowerCase() == category1.toLowerCase() && !styles[i].isShortcut) {
            result.push(styles[i]);
        }
    }
    return result;
}

/* 获取某个样式集合中所有的值 */
const getValuesFromStyles = function (styles) {
    var result = [];
    for (var i = 0; i < styles.length; i++) {
        result.push(styles[i].value);
    }
    return result;
}

/* 获取图表数据列列名称 单柱图类型的图表 */
const getChartDataFieldName = function (fields, fieldName) {
    var tt = fields.find(d => {
        return d.name.toLowerCase() == fieldName.toLowerCase();
    });
    return tt.binding[0].columnName;
}

/* 获取图表数据 单柱图类型的图表 */
const getChartData = function (rowDatas, fields, name, displayCount) {
    // 计算单柱图的显示个数
    let length = rowDatas.length;
    if (displayCount != null && displayCount > -1 && displayCount < length) {
        length = displayCount;
    }

    let result = [];
    let columnName = this.getChartDataFieldName(fields, name);
    for (var i = 0; i < length; i++) {
        var rowData = rowDatas[i];
        result.push(rowData[columnName]);
    }
    return result;
}
/* 获取图标数据  数据对象结构形式 */
const getPieData = function (rowData, fields, measureName, displayName, displayCount) {
    let valName = this.getChartDataFieldName(fields, measureName);
    let disName = this.getChartDataFieldName(fields, displayName);
    let length = rowData.length;
    if (displayCount != null && displayCount > -1 && displayCount < length) {
        length = displayCount;
    }
    let result = [];
    for (let i = 0; i < length; i++) {
        result.push({
            value: rowData[i][valName],
            name: rowData[i][disName]
        });
    }
    return result;
}
/* 获取多指标图表数据 */
const getMultiMeasureChartData = function (rowDatas, fields, fieldName, name) {
    let result = [];
    let onefield = fields.find(d => {
        return d.name.toLowerCase() == fieldName.toLowerCase();
    });
    var oneBinding = onefield.binding.find(bind => {
        return bind.columnName.toLowerCase() == name.toLowerCase();
    });
    let columnName = oneBinding.columnName;
    for (var i = 0; i < rowDatas.length; i++) {
        var rowData = rowDatas[i];
        result.push(rowData[columnName]);
    }

    return result;
}

/* 根据列名获取某一列值集合 */
const getValuesFromColumnName = function (rowDatas, columnName) {
    var result = [];
    for (var i = 0; i < rowDatas.length; i++) {
        var item = rowDatas[i];
        var tempIndex = result.indexOf(item[columnName]); // 返回下标
        if (tempIndex == -1) {
            result.push(item[columnName]);
        }
    }
    return result;
}

/* 获取堆积图指定维度值和图例值对应的指标数据 */
const getStackChartValue = function (rowDatas, legendColumnName, legendValue, dimensionColumnName, dimensionValue, measureColumnName) {
    var result = 0;
    for (var i = 0; i < rowDatas.length; i++) {
        var item = rowDatas[i];
        if (item[legendColumnName] == legendValue && item[dimensionColumnName] == dimensionValue) {
            return result = item[measureColumnName];
        }
    }
    return result;
}

/* 获取堆积图表数据 */
const getStackChartData = function (rowDatas, legendColumnName, legendStyles, dimensionColumnName, dimensionValues, measureColumnName) {
    var result = [];
    for (var i = 0; i < legendStyles.length; i++) {
        var obj = {};
        obj.name = legendStyles[i].category2;
        obj.color = legendStyles[i].value;
        obj.data = [];
        result.push(obj);
        for (var j = 0; j < dimensionValues.length; j++) {
            obj.data.push(getStackChartValue(rowDatas, legendColumnName, obj.name, dimensionColumnName, dimensionValues[j], measureColumnName));
        }
    }
    return result;
}

/* 获取折线图表数据 */
const getLineChartData = function (rowDatas, fields, name, itemSize, itemColor) {
    let result = [];
    let columnName = this.getChartDataFieldName(fields, name);
    for (var i = 0; i < rowDatas.length; i++) {
        var rowData = rowDatas[i];
        var item = {
            value: rowData[columnName],
            symbol: 'circle',
            symbolSize: itemSize,
            itemStyle: {
                normal: {
                    color: itemColor
                }
            }
        };
        result.push(item);
    }
    return result;
}

/* 获取多指标折线图表数据 */
const getMultiMeasureLineChartData = function (rowDatas, fields, fieldName, name, itemSize, itemColor) {
    let result = [];
    let onefield = fields.find(d => {
        return d.name.toLowerCase() == fieldName.toLowerCase();
    });
    var oneBinding = onefield.binding.find(bind => {
        return bind.columnName.toLowerCase() == name.toLowerCase();
    });
    let columnName = oneBinding.columnName;
    for (var i = 0; i < rowDatas.length; i++) {
        var rowData = rowDatas[i];
        var item = {
            value: rowData[columnName],
            symbol: 'circle',
            symbolSize: itemSize,
            itemStyle: {
                normal: {
                    color: itemColor
                }
            }
        };
        result.push(item);
    }
    return result;
}


/* 获取水平位置 */
const getHorizontalPosition = function (position) {
    switch (position) {
        case "顶部":
            return 'center';
        case "底部":
            return 'center';
        case "左部":
            return 'left';
        case "右部":
            return 'right';
    }
}

/* 获取垂直位置 */
const getVerticalPosition = function (position) {
    switch (position) {
        case "顶部":
            return 'top';
        case "底部":
            return 'bottom';
        case "左部":
            return 'middle';
        case "右部":
            return 'middle';
    }
}

/* 获取图例列表的布局朝向 */
const getOrient = function (position) {
    switch (position) {
        case "顶部":
            return 'horizontal';
        case "底部":
            return 'horizontal';
        case "左部":
            return 'vertical';
        case "右部":
            return 'vertical';
    }
};
//将颜色转换成RGBA的形式
const colorRgba = function (color) {
    let val = null;
    if (color.indexOf('#') > -1) {
        if (color.length > 8) {
            let alpha = color.slice(7, 9);
            alpha = +(parseInt(alpha, 16) / 255).toFixed(2);
            val = 'rgba(' + parseInt('0x' + color.slice(1, 3)) + ',' + parseInt('0x' + color.slice(3, 5)) + ',' + parseInt('0x' + color.slice(5, 7)) + ',' + alpha + ')';
        } else {
            val = 'rgba(' + parseInt('0x' + color.slice(1, 3)) + ',' + parseInt('0x' + color.slice(3, 5)) + ',' + parseInt('0x' + color.slice(5, 7)) + ',' + 1 + ')';
        }
    } else {
        val = color;
    }
    return val;
};
/* 处理rowData数据,但fileds里面的binding长度只能为1 */
const changeData = function (rowData, fileds) {
    let dataCollection = [];
    for (let i = 0; i < rowData.length; i++) {
        let obj = {};
        for (let key in rowData[i]) {
            for (let j = 0; j < fileds.length; j++) {
                if (key == fileds[j].binding[0].columnName) {
                    obj[fileds[j].name] = rowData[i][key]
                }
            }
        }
        dataCollection.push(obj)
    }
    return dataCollection;
}

export default {
    deepClone,
    getDataValue,
    getMaxOrMin,
    getColorEndPoint,
    getColor,
    getNumberString,
    getIntegerPart,
    getDecimalPart,
    toThousandsFromIntegerString,
    toThousandsFromNumberString,
    getShadowSetting,
    convertPositionToString,
    convertRotateToFontStyle,
    convertYesNoToBoolean,
    convertLineTypeToString,
    convertFloatValueToPercentValue,
    convertValueStyleToVisibility,
    getStyleValue,
    getStylesFromCategory,
    getValuesFromStyles,
    getChartDataFieldName,
    getChartData,
    getPieData,
    getMultiMeasureChartData,
    getValuesFromColumnName,
    getStackChartValue,
    getStackChartData,
    getLineChartData,
    getMultiMeasureLineChartData,
    getHorizontalPosition,
    getVerticalPosition,
    getOrient,
    colorRgba,
    getStyleObj,
    changeData,
    getTextColor,
    imageStretch,
    getApiFieldNameFromMapping
};