// 常用函数

// 计算元素是否被选中
const isElementSelected = function (selectedArray, element) {
    let selected = selectedArray.find(s => s.id === element.id);
    return selected != undefined;
}

/**
 * 递归设置是否可拖拽属性为不可拖拽
 * @param {*} element 
 */
const setUnDraggable = function (element) {
    if (element.children.length != 0) {
        element.children.forEach(ele => {
            ele.draggable = false;
            setUnDraggable(ele)
        });
    }
    element.draggable = false;
}

/**
 * 设置属性
 * @param {*} element 
 * @param {*属性名称} propertyName 
 * @param {*属性集合} propertyValue 
 */
const setProperty = function (element, propertyName, propertyValue) {
    if (element.childrens.length != 0) {
        element.childrens.forEach(ele => {
            if (ele[propertyName] != undefined) {
                ele[propertyName] = propertyValue;
            }
            setProperty(ele, propertyName, propertyValue);
        });
    }
    if (element[propertyName] != undefined)
        element[propertyName] = propertyValue;
}

/**
 * 生成唯一标识
 */
const getGuid = function () {
    let time = new Date().getTime();
    if (window.performance && typeof window.performance.now === 'function') {
        time += performance.now();
    }
    let guid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        let r = (time + Math.random() * 16) % 16 | 0;
        time = Math.floor(time / 16);
        return (c == 'x' ? r : (r & 0x3 | 0x8)).toString(16);
    });
    return guid;
}



/**
 * 获取图表尺寸
 * @param {layer} element 
 */
const getSize = function (element) {
    if (!element.style) return null;
    return element.style.find(s => s.display == "图表尺寸");

}


/**
 * 获取图表的位置
 * @param {layer} element 
 */
const getPostion = function (element) {
    if (!element.style) return null;
    return element.style.find(s => s.display == "图表位置");
}


/**
 * 获取随机颜色的方法
 * @param alpha 透明度 0-1
 * @returns {string}
 */
const getRandomRGBAColor = function (alpha) {
    let r = Math.round(Math.random() * 255);
    let g = Math.round(Math.random() * 255);
    let b = Math.round(Math.random() * 255);
    if (alpha === undefined) {
        return 'rgba(' + r + ',' + g + ',' + b + ',' + 1 + ')';
    } else {
        return 'rgba(' + r + ',' + g + ',' + b + ',' + alpha + ')';
    }
}


const colors = [
    '#FFFF00FF', '#BDFD00FF', '#72D803FF', '#00A880FF', '#009ACFFF',
    '#9673FFFF', '#DC73FFFF', '#FF7A4DFF', '#FF9326FF', '#FFC926FF'
];

/**
 * 获取随机的16进制颜色
 */
const getRandom16Color = function (index) {
    // let r = Math.round(Math.random() * 255);
    // let g = Math.round(Math.random() * 255);
    // let b = Math.round(Math.random() * 255);
    // return '#' + r.toString(16) + g.toString(16) + b.toString(16);
    return colors[index % 10]

}


/**
 * 判断是否包含标记
 * @param {字符} stringCode 
 */
const isContainMark = function (stringCode) {
    let isContain;
    if (stringCode) {
        stringCode = stringCode.trim();
        isContain = stringCode.indexOf('{') == 0 && stringCode.indexOf('}') == stringCode.length - 1
    }
    return isContain;
}



export default {
    isElementSelected,
    setUnDraggable,
    setProperty,
    getGuid,
    getSize,
    getPostion,
    getRandomRGBAColor,
    getRandom16Color,
    isContainMark
};